# Urban Dictionary
# Copyright (C) 2006-2008 perpleXa
# http://perplexa.ugug.org / #perpleXa on QuakeNet
#
# Redistribution, with or without modification, are permitted provided
# that redistributions retain the above copyright notice, this condition
# and the following disclaimer.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY, to the extent permitted by law; without
# even the implied warranty of MERCHANTABILITY or FITNESS FOR A
# PARTICULAR PURPOSE.
#
# Usage:
#  !ud [id] <term>

# fsck.tcl and http.tcl is included in the zip file
package require fsck 1.10;
package require http;

namespace eval urbandict {
  variable version 1.13;

  # Flood protection (seconds)
  variable antiflood 10;
 
  # Maximum number of lines to output
  variable max_definition_lines 2;
  variable max_example_lines 2;
  
  variable encoding "utf-8";
  variable client "Mozilla/5.0 (Windows NT 5.1; rv:31.0) Gecko/20100101 Firefox/31.0";
  bind pub -|- "!ud" [namespace current]::pub;
  bind pub -|- ".ud" [namespace current]::pub;
  bind pub -|- "!urban" [namespace current]::pub;
  bind pub -|- ".urban" [namespace current]::pub;
  namespace export pub;
}

proc urbandict::getdefinition {definition} {
  variable client;
  ::http::config -useragent $client;

  set url "http://www.urbandictionary.com/define.php?term=[urlencode $definition]";
  if {[catch {http::geturl $url -timeout 20000} token]} {
    return [list 0 "Warning: Couldn't connect to \[$url\]"];
  }
  upvar 0 $token state;
  if {![string equal -nocase $state(status) "ok"]} {
    return [list 0 "Warning: Couldn't connect to \[$url\] (connection $state(status))."];
  }
  foreach {name val} $state(meta) { set meta([string tolower $name]) $val; }
  if {[info exists meta(location)]} {
      set meta(redirect) $meta(location);
  }
  if {[info exists meta(redirect)]} {
     if {[catch {http::geturl $meta(redirect) -timeout 20000} token]} {
        return [list 0 "Warning: Couldn't connect to \[$meta(redirect)\]"];
     }
  }

  set data [http::data $token];
  http::cleanup $token;
  # Those [censored] keep switching between those options:
  #set matches [regexp -all -inline {<div class="def_p">(.*?)</div>} $data];
  #set matches [regexp -all -inline {<td class='text' colspan='2'>(.*?)</td} $data];
  # yet another change...
  if {[regexp -all -inline {<td class=['"]text['"] colspan=['"].['"] id=['"].*?['"]>(.*?)</td} $data] != ""} {
    set matches [regexp -all -inline {<td class=['"]text['"] colspan=['"].['"] id=['"].*?['"]>(.*?)</td} $data];
  } elseif {[regexp -all -inline {<div class=['"]text['"] colspan=['"].['"] id=['"]entry.*?['"]>(.*?)<a class} $data] != ""} {
    set matches [regexp -all -inline {<div class=['"]text['"] colspan=['"].['"] id=['"]entry.*?['"]>(.*?)<a class} $data];
  } elseif {[regexp -all -inline {<a class=['"]add_to_list['"].*?>(.*?)<a class} $data] != ""} {
    set matches [regexp -all -inline {<a class=['"]add_to_list['"].*?>(.*?)<a class} $data];
  } elseif {[regexp -all -inline {<div class=['"]word['"]>(.*?)<div class=['"]share['"]} $data] != ""} {
    set matches [regexp -all -inline {<div class=['"]word['"]>(.*?)<div class=['"]share['"]} $data];
  } elseif {[regexp -all -inline {<div class=['"]row['"]>(.*?)<div class=['"]def-footer['"]} $data] != ""} {
    set matches [regexp -all -inline {<div class=['"]row['"]>(.*?)<div class=['"]def-footer['"]} $data];
  } elseif {[regexp -all -inline {<div class=['"]def-header['"]>(.*?)<div class=['"]def-footer['"]} $data] != ""} {
    set matches [regexp -all -inline {<div class=['"]def-header['"]>(.*?)<div class=['"]def-footer['"]} $data];
  } else {
    set matches "";
  }
 
  set list [list];
  foreach {null match} $matches {
    set definition "n/a";
    if {[regexp -nocase -- {<div class=['"]definition['"]>(.*?)</div>} $match -> definition]} {
    } elseif {[regexp -nocase -- {<div class=['"]meaning['"]>(.*?)</div>} $match -> definition]} {
    }
    regsub -all {[\r\n\s\t]+} $definition " " definition;
	regsub -all {} $definition {-} definition;
    regsub -all {<[^>]*?>} [string trim [decode $definition]] "" definition;
    set example "";
    if {[regexp -nocase -- {<div class=['"]example['"]></div>} $match - -]} {
      set example "None";
    } elseif {[regexp -nocase -- {<div class=['"]example['"]>_+</div>} $match - -]} {
      set example "None";
    } elseif {[regexp -nocase -- {<div class=['"]example['"]>(.*?)</div>} $match -> example] && $example != "None"} {
      regsub -all {[\r\n\s\t]+} [string trim $example] " " example;
      regsub -all {} $example {-} example;
      regsub -all {<[^>]*?>} [string trim [decode $example]] "" example;
    } else {
      set example "None";
    }
    lappend list "$definition[expr {$example == "" ? "" : " \[\($example\)\]"}]";
  }
  return [concat [llength $list] $list];
}

proc urbandict::urlencode {i} {
  variable encoding
  set index 0;
  set i [encoding convertto $encoding $i]
  set length [string length $i]
  set n ""
  while {$index < $length} {
    set activechar [string index $i $index]
    incr index 1
    if {![regexp {^[a-zA-Z0-9]$} $activechar]} {
      append n %[format "%02X" [scan $activechar %c]]
    } else {
      append n $activechar
    }
  }
  return $n
}

proc urbandict::pub {nick host hand chan argv} {
  variable flood; variable antiflood; variable max_definition_lines; variable max_example_lines;
  if {![info exists flood($chan)]} { set flood($chan) 0; }
  if {![string compare $argv ""]} {
    puthelp "NOTICE $nick :Usage: !ud \[id\] <definition>";
    return 1;
  }
  if {[string is digit -strict [getword $argv 0]]} {
    if {[splitline $argv cargv 2]!=2} {
      puthelp "NOTICE $nick :Usage: !ud \[id\] <definition>";
      return 1;
    }
    set id [lindex $cargv 0];
    set argv [lindex $cargv 1];
    if {!$id} {
      set id 1;
    }
  } else {
    set id 1;
  }

  if {[unixtime] - $flood($chan) <= $antiflood} { return 0; }
  set flood($chan) [unixtime];

  set definitions [getdefinition $argv];
  set count [lindex $definitions 0];
  if {!$count} {
    puthelp "PRIVMSG $chan :Nothing found for \"$argv\".";
    return 1;
  } elseif {$id > $count} {
    puthelp "PRIVMSG $chan :Only $count results found for \"$argv\".";
    return 1;
  }
  set definition [lindex $definitions $id];
  regexp -- {(.*?) \[\((.*?)\)\]} $definition - define example;
  if {[string length $definition] <= 400} {
    puthelp "PRIVMSG $chan :\002Definition \[$id/$count\]:\002 $define";
    if {$example != "None"} {
      puthelp "PRIVMSG $chan :\002Example \[$id/$count\]:\002 $example";
    }
    return 0;
  }
  set skip 0;
  set define [splitmsg $define];
  set example [splitmsg $example];
  set required_d_lines [llength $define];
  set required_e_lines [llength $example];
  set tcount 0;
  foreach line $define {
    if {$tcount == 0} {
      puthelp "PRIVMSG $chan :\002Definition \[$id/$count\]:\002 $line";
      incr tcount;
	} else {
      puthelp "PRIVMSG $chan :$line";	
	}
    if {[incr skip] == $max_definition_lines && $required_d_lines != $max_definition_lines} {
      # It's really pointless to send this message and break the line limits:
      # puthelp "PRIVMSG $chan :Truncated output to $max_definition_lines of $required_d_lines lines.";
      break;
    }
  }
  if {$example != "None"} {
    set skip 0;
    set tcount 0;
    foreach line $example {
      if {$tcount == 0} {
        puthelp "PRIVMSG $chan :\002Example: \[$id/$count\]:\002 $line";
        incr tcount;
	  } else {
        puthelp "PRIVMSG $chan :$line";	
      }
      if {[incr skip] == $max_example_lines && $required_e_lines != $max_example_lines} {
      # It's really pointless to send this message and break the line limits:
      # puthelp "PRIVMSG $chan :Truncated output to $max_example_lines of $required_e_lines lines.";
        break;
      }
    }
  }
  return 0;
}

proc urbandict::decode {content} {
  if {![string match *&* $content]} {
    return $content;
  }
  set escapes {
    &nbsp; \x20 &quot; \x22 &amp; \x26 &apos; \x27 &ndash; \x2D
    &lt; \x3C &gt; \x3E &tilde; \x7E &euro; \x80 &iexcl; \xA1
    &cent; \xA2 &pound; \xA3 &curren; \xA4 &yen; \xA5 &brvbar; \xA6
    &sect; \xA7 &uml; \xA8 &copy; \xA9 &ordf; \xAA &laquo; \xAB
    &not; \xAC &shy; \xAD &reg; \xAE &hibar; \xAF &deg; \xB0
    &plusmn; \xB1 &sup2; \xB2 &sup3; \xB3 &acute; \xB4 &micro; \xB5
    &para; \xB6 &middot; \xB7 &cedil; \xB8 &sup1; \xB9 &ordm; \xBA
    &raquo; \xBB &frac14; \xBC &frac12; \xBD &frac34; \xBE &iquest; \xBF
    &Agrave; \xC0 &Aacute; \xC1 &Acirc; \xC2 &Atilde; \xC3 &Auml; \xC4
    &Aring; \xC5 &AElig; \xC6 &Ccedil; \xC7 &Egrave; \xC8 &Eacute; \xC9
    &Ecirc; \xCA &Euml; \xCB &Igrave; \xCC &Iacute; \xCD &Icirc; \xCE
    &Iuml; \xCF &ETH; \xD0 &Ntilde; \xD1 &Ograve; \xD2 &Oacute; \xD3
    &Ocirc; \xD4 &Otilde; \xD5 &Ouml; \xD6 &times; \xD7 &Oslash; \xD8
    &Ugrave; \xD9 &Uacute; \xDA &Ucirc; \xDB &Uuml; \xDC &Yacute; \xDD
    &THORN; \xDE &szlig; \xDF &agrave; \xE0 &aacute; \xE1 &acirc; \xE2
    &atilde; \xE3 &auml; \xE4 &aring; \xE5 &aelig; \xE6 &ccedil; \xE7
    &egrave; \xE8 &eacute; \xE9 &ecirc; \xEA &euml; \xEB &igrave; \xEC
    &iacute; \xED &icirc; \xEE &iuml; \xEF &eth; \xF0 &ntilde; \xF1
    &ograve; \xF2 &oacute; \xF3 &ocirc; \xF4 &otilde; \xF5 &ouml; \xF6
    &divide; \xF7 &oslash; \xF8 &ugrave; \xF9 &uacute; \xFA &ucirc; \xFB
    &uuml; \xFC &yacute; \xFD &thorn; \xFE &yuml; \xFF
  };
  set content [string map $escapes $content];
  set content [string map [list "\]" "\\\]" "\[" "\\\[" "\$" "\\\$" "\\" "\\\\"] $content];
  regsub -all -- {&#([[:digit:]]{1,5});} $content {[format %c [string trimleft "\1" "0"]]} content;
  regsub -all -- {&#x([[:xdigit:]]{1,4});} $content {[format %c [scan "\1" %x]]} content;
  regsub -all -- {&#?[[:alnum:]]{2,7};} $content "?" content;
  return [subst $content];
}

putlog "Script loaded: Urban Dictionary v$urbandict::version by perpleXa";
